;---------------------------------------------------
; TI-Menu.inc
; Re-usable menu code for CrASH.
; (c) 1999 Vasantha Crabb <_merlin@operamail.com>
;
; See file TI-Menu.txt for more information.
;---------------------------------------------------

.module     TIMenu

; These macros address the menu data structure
#DEFINE MENU_LEN        ix+$00
#DEFINE MENU_DEFAULT    ix+$01
#DEFINE MENU_SEL        ix+$02
#DEFINE MENU_TITLE      ix+$03

;---------------------
; SUB for handling a menu
;---------------------

RunMenu:
    res     1,(iy+$0D)          ; Don't mess with text memory
    push    hl                  ; Get address menu structure it into ix
    pop     ix

    push    hl
    ROM_CALL(CLEARLCD)          ; Clear the text screen
    pop     hl

    ld      b,(hl)              ; Get the number of items in b

    inc     hl                  ; Point hl at the title
    inc     hl
    inc     hl

    ld      de,$0000            ; Move to the top left corner of LCD
    ld      (CURSOR_ROW),de
    set     3,(iy+$05)          ; White on black for title
    ROM_CALL(D_ZT_STR)          ; Display the title

    ld      c,1
    res     3,(iy+$05)          ; Black on white for the options
_MenuDispLoop:
    ld      a,c                 ; Put item number in a
    call    _MenuDispItemNo     ; Show item number

    ROM_CALL(D_ZT_STR)          ; Display the item's text
    inc     c                   ; Move down a line
    djnz    _MenuDispLoop       ; Loop

    ld      a,(MENU_DEFAULT)    ; Start on default item
    call    _MenuNewItem        ; Get the marker onto the screen

_MenuHandLoop:
    call    CR_KHAND            ; Get a key and handle it

    cp      K_QUIT              ; The user can exit at any time
    call    z,ClearOut

    cp      K_UP                ; Select previous menu item
    call    z,_MenuUpHandler

    cp      K_DOWN              ; Select next menu item
    call    z,_MenuDwnHandler

    cp      K_ENTER             ; User's made up their mind
    jr      z,_MenuRetItem

    cp      K_CLEAR             ; User wants out
    jr      z,_MenuRetZero

    cp      K_1                 ; See if they've hit an item number
    jr      c,_MenuHandLoop
    sub     K_0                 ; Change keycode to item number
    ld      b,a                 ; Save this for now
    ld      a,(MENU_LEN)        ; See if it's on the menu
    cp      b
    jr      c,_MenuHandLoop     ; Loop if it isn't
    ld      a,b                 ; Otherwise we return
    ret

_MenuRetItem:
    ld      a,(MENU_SEL)        ; Put the selection in a
    ret                         ; Return

;---------------------
; SUB for handling DOWN key on a menu
;---------------------

_MenuDwnHandler:
    ld      a,(MENU_SEL)        ; Get the current menu item
    cp      (MENU_LEN)          ; Return if we're on the last item
    jr      z,_MenuRetZero

    push    af                  ; Save item number
    call    _MenuDispItemNo     ; Show item number (normal text)

    pop     af                  ; Restore item number
    inc     a                   ; Increment menu item
_MenuNewItem:                   ; For code re-use
    ld      (MENU_SEL),a        ; Store the new menu item

    set     3,(iy+$05)          ; White text on black
    call    _MenuDispItemNo     ; Show item number (inverse text)
    res     3,(iy+$05)          ; Black text on white

_MenuRetZero:
    xor     a
    ret                         ; Return

;---------------------
; SUB for handling UP key on a menu
;---------------------

_MenuUpHandler:
    ld      a,(MENU_SEL)        ; Get the current menu item
    cp      1                   ; Return if we're on the first item
    jr      z,_MenuRetZero

    push    af                  ; Save item number
    call    _MenuDispItemNo     ; Show item number (normal text)

    pop     af                  ; Restore item number
    dec     a                   ; Decrement menu item
    jr      _MenuNewItem        ; Re-use code

;---------------------
; SUB for showing menu item number
;---------------------

_MenuDispItemNo:
    ld      d,$00               ; Cursor column 1
    ld      e,a                 ; Cursor row a
    ld      (CURSOR_ROW),de
    add     a,'0'               ; Convert item number to character
    ROM_CALL(TX_CHARPUT)        ; Put the character on the screen
    ld      a,':'               ; Print a colon
    ROM_CALL(TX_CHARPUT)
    ret                         ; Return

#IFNDEF ClearOut

ClearOut:
    ld      SP,(OldSP)          ; Kill excess stack items
    ret                         ; Return to shell

#ENDIF
